# -*- coding: utf-8 -*-
"""
Created on Mon Apr 17 18:54:24 2023

@author: chman
"""

#!/usr/bin/env python
# coding: utf-8

# # Data Structure

# ![image-3.png](attachment:image-3.png)

#%% Structure

# 0 Data Preparation

#   0.1 Deal Data
#   0.2 Accounting Data
#   0.3 Return Data

# 1 Regressions - Why an ESG score matters
 
#   1.1 Firms with ESG scores participate in larger transactions and have a larger deal size
#   1.2 Bidders with an ESG Score are more likely to bid for targets with an ESG Score
#   1.3 Acquirors with an ESG score are more likely to win a bidding contest

# 2 Regressions - Why a high ESG score matters

#   2.1 Firms with high ESG scores participate in larger transactions and have larger deal sizes
#   2.2 Bidders with a high ESG score are more likely to bid for targets with a high ESG score
#   2.3 Acquirors with a higher ESG score are more likely to win a bidding contest

# 3 Regressions - ESG targets and M&A performance

# 4 Descriptive Statistics
#   4.1 ESG statistics
#   4.2 Winner & Loser statistics    
#   4.3 Industry statistics
#   4.4 Deal statistics

# In[1]:

# standard packages
import pandas as pd
import numpy as np

# plotting
import seaborn as sns
import matplotlib.pyplot as plt
import matplotlib.ticker as ticker


# ignore warnings
import warnings 
warnings.filterwarnings("ignore")

# regressions
import statsmodels.api as sm
import scipy.stats as stats
from statsmodels.iolib.summary2 import summary_col


# In[2]:

## import data
# set path to import Refinitiv Data
file1 = 'C:/Users/chman/Desktop/Dokumente/02 MSc International Finance/Masterthesis/Final/Data/Refinitiv_Data.xlsx'

# set path to import CUSIP to PERMNO conversion file
file3 = 'C:/Users/chman/Desktop/Dokumente/02 MSc International Finance/Masterthesis/Final/Data/PERMNO_Converter.xlsx'

# set path to import CRSP monthly data
file5 = 'C:/Users/chman/Desktop/Dokumente/02 MSc International Finance/Masterthesis/Final/Data/CRSP_monthly_data.csv'

# set path to import annual Compustat data
file7 = 'C:/Users/chman/Desktop/Dokumente/02 MSc International Finance/Masterthesis/Final/Data/Compustat_Data_annual.csv'

## save data
# set path where to save first .txt file
file2 = 'C:/Users/chman/Desktop/Dokumente/02 MSc International Finance/Masterthesis/Final/Data/PERMNO_Codes.txt'

# set path where to save second .txt file
file4 = 'C:/Users/chman/Desktop/Dokumente/02 MSc International Finance/Masterthesis/Final/Data/PERMNO_Codes_prepared.txt'

# set path where to save third .txt file
file6 = 'C:/Users/chman/Desktop/Dokumente/02 MSc International Finance/Masterthesis/Final/Data/CUSIP_Codes_prepared.txt'

#%% 
#################################### 0 Data Preparation

#%%
#################################### 0.1 Deal Data

#%%

# read by default 1st sheet of an excel file
df1 = pd.read_excel(file1)

#%%
# shrink dataset for 2002, as this is the first year for which ESG Data is available
# After or equal to 2002

df1['Date Announced'] = pd.to_datetime(df1['Date Announced'])

df1 = df1[df1['Date Announced'].dt.year >= 2002]

#%%

# drop repeated bids by same bidder

# Group the data by bidder and target and sort each group by "announcement date"
df_sorted = df1.groupby(['Acquiror 6-digit CUSIP', 'Target 6-digit CUSIP']).apply(lambda x: x.sort_values('Date Announced'))

df_sorted = df_sorted.reset_index(drop=True)

# Finally, drop the duplicate bids by keeping only the first bid for each group
df1 = df_sorted.drop_duplicates(subset=['Acquiror 6-digit CUSIP', 'Target 6-digit CUSIP'], keep='first')

# In[4]:

# add a unique identifier per deal to deal information
# consists of Target

df1['deal_identifier'] = df1['Target 6-digit CUSIP']
df1['Percentage of Cash'] = pd.to_numeric(df1['Percentage of Cash'])
df1['Percentage of Stock'] = pd.to_numeric(df1['Percentage of Stock'])
df1['Percentage of Other'] = pd.to_numeric(df1['Percentage of Other'])
df1['Deal Value'] = pd.to_numeric(df1['Deal Value'])


# In[199]:

# Analyse data

df1['Winner_Dummy'] = np.where(df1['Deal Status'] == "Completed", 1, 0)

initial_bids_0 = int(df1[['SDC Deal No']].count())
initial_bidders_0 = df1['Acquiror 6-digit CUSIP'].unique()
initial_bidders_0 = len(initial_bidders_0)
initial_w_0 = int(df1[['Winner_Dummy']].sum())
initial_l_0 = int(initial_bidders_0-initial_w_0)
initial_c_0 = df1['deal_identifier'].unique()
initial_c_0 = len(initial_c_0)

df1['Winner_Dummy'] = np.where(df1['Deal Status'] == 'Completed', 1, 0)
df1['Friendly_Dummy'] = np.where(df1['Deal Attitude'] == 'Friendly', 1, 0)

#%%

# export .txt file with the relevant CUSIPs for the Acquiror to feed into WRDS

np.savetxt(file2 , df1['Acquiror 6-digit CUSIP'], fmt= '%s')


#%%
#################################### 0.2 Accounting Data

#%%

# download a .xlsx file from WRDS tools with the WRDS path:  
# Home > Get Data > CRSP > Annual Update > Tools > Translate to PERMCO/PERMNO

# Get all of the variables Company Name (COMNAM), Ticker (TICKER), Exchange Code (EXCHCD) and NCUSIP (NCUSIP)
# Date Format: DDMMYY10 (e.g. 25/07/1984)
# Timeframe: Jan 1982 - Dec 2021

df8 = pd.read_excel(file3)
df8.rename(columns = {'Names CUSIP':'NCUSIP'}, inplace = True)
df8['Acquiror 6-digit CUSIP'] = df8['NCUSIP'].str[0:6]

df8.rename(columns = {'Date of Observation. NAMEDT of NAMES events; ex-distribution date (EXDT) of DISTRIBUTION event; delisting date(DLSTDT) of DELISTING event; shares observation date (SHRSDT) of SHARES OUTSTANDING event; NASDAQ traits date (TRTSDT) of NASDAQ INFORMATION event': 'Date'}, inplace = True)

df8.groupby('Acquiror 6-digit CUSIP')

# In[202]:

# Convert PERMNO & Date to str to be able to perform function below
df8['PERMNO'] = df8['PERMNO'].apply(str)
df8['Date'] = df8['Date'].apply(str)

# create columns with multiple values for PERMNO & Date for each acquiror 
df_permno = df8.groupby('Acquiror 6-digit CUSIP').PERMNO.apply(lambda x: ", ".join(x)).reset_index()
df_date = df8.groupby('Acquiror 6-digit CUSIP').Date.apply(lambda x: ", ".join(x)).reset_index()

# add columns to the acquiror CUSIP
df9 = pd.merge(df_permno, df_date, on = 'Acquiror 6-digit CUSIP', how = 'inner')


# In[203]:

# Split PERMNO Codes and label them
df10 = df9['PERMNO'].str.split(',', expand=True).add_prefix('PERMNO').fillna('')
df9 = df9.join(df10)
df9.drop('PERMNO', axis = 1, inplace = True)

# Split Dates of PERMNO Codes and label them
df11 = df9['Date'].str.split(',', expand=True).add_prefix('Date').fillna('')
df9 = df9.join(df11)
df9.drop('Date', axis = 1, inplace = True)

# convert dates back into timestamp format to perform calculations with them later on
df9.Date0 = pd.to_datetime(df9.Date0)
df9.Date1 = pd.to_datetime(df9.Date1)
df9.Date2 = pd.to_datetime(df9.Date2)
df9.Date3 = pd.to_datetime(df9.Date3)
df9.Date4 = pd.to_datetime(df9.Date4)
df9.Date5 = pd.to_datetime(df9.Date5)

# In[204]:

# create new dataframe in which every bid is matched to the respective PERMNO Codes

df10 = df1[['SDC Deal No', 'Acquiror 6-digit CUSIP', 'Date Announced']]

# left join the collected data for PERMNO Codes and respective Dates to different Bids
df11 = pd.merge(df10, df9, on = 'Acquiror 6-digit CUSIP', how = 'left')

# clean data and drop bids for which the acquirer has no PERMNO Code 
df11 = df11.replace(r'^s*$', float('NaN'), regex = True) # replace empty PERMNO cells with NaN
df11.dropna(subset = ['PERMNO0'], inplace = True)
df11.reset_index(drop=True)

# In[205]:

# calculate the differences between the Effective Deal Date and the PERMNO Code Date
df11['Difference0'] = df11['Date Announced'] - df11['Date0']
df11['Difference1'] = df11['Date Announced'] - df11['Date1']
df11['Difference2'] = df11['Date Announced'] - df11['Date2']
df11['Difference3'] = df11['Date Announced'] - df11['Date3']
df11['Difference4'] = df11['Date Announced'] - df11['Date4']
df11['Difference5'] = df11['Date Announced'] - df11['Date5']

# convert the date-difference in days into integer

df11['Difference0'] = pd.to_numeric(df11['Difference0'].dt.days)
df11['Difference1'] = pd.to_numeric(df11['Difference1'].dt.days)
df11['Difference2'] = pd.to_numeric(df11['Difference2'].dt.days)
df11['Difference3'] = pd.to_numeric(df11['Difference3'].dt.days)
df11['Difference4'] = pd.to_numeric(df11['Difference4'].dt.days)
df11['Difference5'] = pd.to_numeric(df11['Difference5'].dt.days)

df12 = df11.iloc[:,-6:]

# In[206]:

# write NaN for values below 30
df12[df12 < 29] = np.nan


# In[207]:

df12['Difference0'] = df12['Difference0'].astype('float')
df12['Difference1'] = df12['Difference1'].astype('float')
df12['Difference2'] = df12['Difference2'].astype('float')
df12['Difference3'] = df12['Difference3'].astype('float')
df12['Difference4'] = df12['Difference4'].astype('float')
df12['Difference5'] = df12['Difference5'].astype('float')

# In[208]:

# get array that keeps the lowest value per row
x = df12.idxmin(axis=1, skipna=True)

# In[209]:

df11['min_index'] = x.str[10:11]

df11['min_index'] = df11['min_index'].astype('float')
# df11['min_index'] = pd.to_numeric(df11['min_index'], errors='coerce')

# drop all of the bids that have no PERMNO
df11.dropna(subset = ['min_index'], inplace = True)

# In[210]:

# create function that assigns the final PERMNO Code that is relevant for each transaction to the dataframe

def f(row):
    if row['min_index'] == 0:
        val = row['PERMNO0']
    elif row['min_index'] == 1:
        val = row['PERMNO1']
    elif row['min_index'] == 2:
        val = row['PERMNO2']
    elif row['min_index'] == 3:
        val = row['PERMNO3']
    elif row['min_index'] == 4:
        val = row['PERMNO4']  
    elif row['min_index'] == 5:
        val = row['PERMNO5']
    else:
        val = 'NaN'
    return val

# In[211]:

# add column that contains the final PERMNO code to work with

df11['final_PERMNO'] = df11.apply(f, axis=1)

# convert into integer
df11['final_PERMNO'] = df11['final_PERMNO'].astype('int')

# drop index
df11 = df11.reset_index(drop=True)

# In[212]:

a = df11.iloc[:,-1:] # grab final_PERMNO Column
b = df11.iloc[:,0:3] # grab first three columns
df13 = pd.concat([a,b], axis = 1)

# In[213]:

# merge the final PERMNO codes to the deal data
    
df14 = pd.merge(df13, df1, on = 'SDC Deal No', how = 'inner')
df14.rename(columns={'Acquiror 6-digit CUSIP_x':'Acquiror 6-digit CUSIP'}, inplace = 'True')

#%%

#%%
# Add ESG Dummy
df14['ESG_Dummy'] = df14['ESG Acquiror'].notnull().astype(int)


# In[214]:

PERMNO_bids = int(df14[['SDC Deal No']].count())
PERMNO_bidders = df14['final_PERMNO'].unique()
PERMNO_bidders = len(PERMNO_bidders)
PERMNO_w = int(df14[['Winner_Dummy']].sum())
PERMNO_l = int(PERMNO_bidders-PERMNO_w)
PERMNO_c = df14['deal_identifier'].unique()
PERMNO_c = len(PERMNO_c)


#%%
   
#WINNER LOSER ESG Difference

ESG_stats_large_sample_winners = df14[df14['Winner_Dummy'] == 1]
ESG_stats_large_sample_losers = df14[df14['Winner_Dummy'] == 0]

ESG_stats_large_sample_winners = pd.DataFrame(ESG_stats_large_sample_winners['ESG Acquiror'].describe())
ESG_stats_large_sample_winners.rename(columns={'ESG Acquiror': 'ESG Score Winners'}, inplace = True)

ESG_stats_large_sample_losers = pd.DataFrame(ESG_stats_large_sample_losers['ESG Acquiror'].describe())
ESG_stats_large_sample_losers.rename(columns={'ESG Acquiror': 'ESG Score Losers'}, inplace = True)

ESG_stats_large_sample = pd.concat([ESG_stats_large_sample_winners, ESG_stats_large_sample_losers], axis = 1)
 
    
#%%

# export .txt file with the relevant PERMNOs for the Acquiror to feed into WRDS for return data

np.savetxt(file4 , df14['final_PERMNO'], fmt= '%s')

# In[216]:

# merge the final PERMNO with the 8-digit CUSIP from a prior table
# reason: to obtain the Accounting data from Compustat 8-digit CUSIPs must be used
df14.rename(columns={'final_PERMNO':'PERMNO'}, inplace = 'True')
df8['PERMNO'] = df8['PERMNO'].astype('int32')
df14['SDC Deal No'] = df14['SDC Deal No'].astype('str')


df_CUSIP = pd.merge(df8,df14, on = 'PERMNO', how = 'left' )
df14.rename(columns={'PERMNO':'final_PERMNO'}, inplace = 'True')


# In[217]:

# export .txt file with the relevant NCUSIPs for the Acquiror to feed into WRDS

np.savetxt(file6 , df_CUSIP['NCUSIP'], fmt= '%s')

# In[24]:

# download a .csv file from Compustat with the WRDS path:  
# Home > Get Data > Compustat - Capital IQ > Compustat > North America > Fundamentals Annual
# Date Format: DDMMYY10 (e.g. 25/07/1984)

# Get all of the annual variables AT, SALE, DT, SEQ, TXDITC, PSTKL, PSTKRV, UPSTK, CSHO, PRCC_F, OIBDP, PPEGT
# Timeframe: Jan 1999 - Dec 2021

# In[218]:

# read annual accounting data

df25 = pd.read_csv(file7)
df25.rename(columns = {'datadate':'date'}, inplace = True)
df25['Acquiror 6-digit CUSIP'] = df25['cusip'].str[0:6]
df25['NCUSIP'] = df25['cusip'].str[0:8]

df25.date = pd.to_datetime(df25.date)

# In[219]:

# get the PERMNO to the Accounting data
# in WRDS a query for Compustat is only possible with NCUSIP

#convert final_PERMNO into string, to avoid datatype conflicts later on

df13['final_PERMNO'] = df13['final_PERMNO'].astype(str)

df13['SDC Deal No'] = df13['SDC Deal No'].astype(str)
df13['Acquiror 6-digit CUSIP'] = df13['Acquiror 6-digit CUSIP'].astype(str)
df25['Acquiror 6-digit CUSIP'] = df25['Acquiror 6-digit CUSIP'].astype(str)

# add PERMNO code

df22 = df25.merge(df13, on=['Acquiror 6-digit CUSIP'], how='left').fillna('')

# In[220]:

# Naming for Annual Accounting Data according to WRDS Codes

df22.rename(columns = {'at':'AT'}, inplace = True) # 
df22.rename(columns = {'sale':'SALE'}, inplace = True) #
df22.rename(columns = {'dt':'DT'}, inplace = True) # 
df22.rename(columns = {'seq':'SEQ'}, inplace = True) #
df22.rename(columns = {'txditc':'TXDITC'}, inplace = True) #
df22.rename(columns = {'pstkl':'PSTKL'}, inplace = True) #
df22.rename(columns = {'pstkrv':'PSTKRV'}, inplace = True) #
df22.rename(columns = {'upstk':'UPSTK'}, inplace = True) #
df22.rename(columns = {'csho':'CSHO'}, inplace = True) #
df22.rename(columns = {'prcc_f':'PRCC_F'}, inplace = True) #
df22.rename(columns = {'oibdp':'OIBDP'}, inplace = True) #
df22.rename(columns = {'ppegt':'PPEGT'}, inplace = True) #

df22.rename(columns = {'SDC Deal No_y':'SDC Deal No'}, inplace = True) # 


# In[257]:
    
## Merge Accounting Data and Bidder Data 

# merge deal and accounting data
    
# has to always be merged with SDC Deal No
# reason: if there is a firm with multiple bids, the accounting data has to be matched to the date of the bid and not the firm

df22['Acquiror 6-digit CUSIP'] = df22['Acquiror 6-digit CUSIP'].astype(str)
df22['Acquiror 6-digit CUSIP'] = df22['Acquiror 6-digit CUSIP'].str.strip()

df14['Acquiror 6-digit CUSIP'] = df14['Acquiror 6-digit CUSIP'].astype(str)
df14['Acquiror 6-digit CUSIP'] = df14['Acquiror 6-digit CUSIP'].str.strip()


df22['SDC Deal No'] = df22['SDC Deal No'].astype(str)
df22['SDC Deal No'] = df22['SDC Deal No'].str.strip()

df14['SDC Deal No'] = df14['SDC Deal No'].astype(str)
df14['SDC Deal No'] = df14['SDC Deal No'].str.strip()

#%%
df24 = pd.merge(df22,df14, on=['SDC Deal No'], how='outer').fillna('')

#%%
#####################################################################################################################

#%%
df24 = df24.drop("Winner_Dummy", axis=1)
df24['Winner_Dummy'] = np.where(df24['Deal Status'] == 'Completed', 1, 0)
df24['Friendly_Dummy'] = np.where(df24['Deal Attitude'] == 'Friendly', 1, 0)
df24['Industry_Dummy'] = np.where(df24['Target Macro Industry'] == df24['Acquiror Macro Industry'], 1, 0)

# In[258]:

# Grab year from Announcement Date of the bid and relevant year for the accounting data

df24['Announcement Year'] = df24['Date Announced'].astype(str)
df24['Announcement Year'] = df24['Announcement Year'].str[:4]

# In[259]:

df30 = df24

# merge the year of the announcement date of the bid with the accounting data from one year before
df30['fyear'] = pd.to_numeric(df30['fyear'], errors='coerce')

# subtract one from fiscal year to be able to only keep accounting data prior to the announcement of the bid
df30['fyear'] =  df30['fyear'] - 1
df30['fyear'] =  df30['fyear'].astype(str)
df30['fyear'] =  df30['fyear'].str[:4]

#%% filter in the merged dataset to only keep relevant observations

# delete all observations for which the year is wrong
df30 = df30[df30['Announcement Year'] == df30['fyear']]

# delete Deal duplicates

# Identify duplicated deals from merge in a SDC Deal No column
duplicates = df30.duplicated(subset=['SDC Deal No'], keep=False)

# Keep only the first occurrence of each unique value
df30 = df30.drop_duplicates(subset=['SDC Deal No'], keep='first')


# In[260]:

df30.fillna(value=np.nan, inplace=True)

# convert variables to numeric format

df30['ESG Acquiror'] = pd.to_numeric(df30['ESG Acquiror'], errors='coerce')
df30['E Score'] = pd.to_numeric(df30['E Score'], errors='coerce')
df30['S Score'] = pd.to_numeric(df30['S Score'], errors='coerce')
df30['G Score'] = pd.to_numeric(df30['G Score'], errors='coerce')

df30['Target E Score'] = pd.to_numeric(df30['Target E Score'], errors='coerce')


#%%

## Calculate Control Variables

df30['DT'] = pd.to_numeric(df30['DT'])
df30['CSHO'] = pd.to_numeric(df30['CSHO'])
df30['PRCC_F'] = pd.to_numeric(df30['PRCC_F'])
df30['SEQ'] = pd.to_numeric(df30['SEQ'])
df30['TXDITC'] = pd.to_numeric(df30['TXDITC'])
df30['PSTKRV'] = pd.to_numeric(df30['PSTKRV'])
df30['OIBDP'] = pd.to_numeric(df30['OIBDP'])
df30['PPEGT'] = pd.to_numeric(df30['PPEGT'])
df30['AT'] = pd.to_numeric(df30['AT'], errors='coerce')
df30['SALE'] = pd.to_numeric(df30['SALE'], errors='coerce')

#%%

## Sale  (log)

df30['Sales'] = df30['SALE'].apply(lambda x: np.log(x) if x > 0 else 0)

#%%

## Book Leverage

df30['Book Leverage'] = (df30['DT'] / df30['AT'])

# In[264]:

# Market Value (log)
# careful: create a new variable, otherwise calculations with Market Leverage and Q go wrong

df30['MV'] = df30['AT'] + (df30['CSHO'] * df30['PRCC_F']) - ( df30['SEQ'] + df30['TXDITC'] - df30['PSTKRV'])
df30['ln(MV)'] = df30['MV'].apply(np.log)


#%%

# Deal Value (log)

df30['Deal Value'] = df30['Deal Value'].apply(lambda x: np.log(x) if x > 0 else np.nan)


# In[265]:

# Market Leverage

df30['Market Leverage'] = (df30['DT'] / df30['MV'])

# In[266]:

# Tobin's Q (Ratio of MV to BV of Assets)

df30['Q'] = df30['MV'] / df30['AT']

# In[267]:

# Operating Income

df30['OpIncome'] = df30['OIBDP'] / df30['AT']

#%%

df30['PPE'] = df30['PPEGT'] / df30['AT']

#%% 
# Count the number of observations with Accounting Data

df_acc = df30.dropna(subset=['Deal Value','OpIncome','Deal Participation','Market Leverage', 'Q','Friendly_Dummy', 'Industry_Dummy'])

acc_bids = int(df_acc[['SDC Deal No']].count())
acc_bidders = df_acc['final_PERMNO_x'].unique()
acc_bidders = len(acc_bidders)
acc_w = int(df_acc[['Winner_Dummy']].sum())
acc_l = int(acc_bidders-acc_w)
acc_c = df_acc['deal_identifier'].unique()
acc_c = len(acc_c)


#%% Add time FE

df30['fe_year'] = pd.factorize(df30['Announcement Year'])[0]

# Create a set of dummy variables for the contest fixed effects
dummy_year = pd.get_dummies(df30['fe_year'], prefix='year', drop_first=True)

# Add the dummy variables to the original DataFrame
df30 = pd.concat([df30, dummy_year], axis=1)

df30['ESG Target'] = pd.to_numeric(df30['ESG Target'])


#%%
# Preparations for Regressions

# exclude values without all of the dummies

df_ESG = df30.dropna(subset=['ESG Acquiror','Deal Value','OpIncome','Deal Participation','Market Leverage','Q','Friendly_Dummy', 'Industry_Dummy'])
df_E = df30.dropna(subset=['E Score','Deal Value','OpIncome','Deal Participation','Market Leverage','Q','Friendly_Dummy', 'Industry_Dummy'])
df_S = df30.dropna(subset=['S Score','Deal Value','OpIncome','Deal Participation','Market Leverage','Q','Friendly_Dummy', 'Industry_Dummy'])
df_G = df30.dropna(subset=['G Score','Deal Value','OpIncome','Deal Participation','Market Leverage','Q','Friendly_Dummy', 'Industry_Dummy'])


#%%
# Count the number of observations with ESG and Accounting Data

esg_bids = int(df_ESG[['SDC Deal No']].count())
esg_bidders = df_ESG['final_PERMNO_x'].unique()
esg_bidders = len(esg_bidders)
esg_w = int(df_ESG[['Winner_Dummy']].sum())
esg_l = int(esg_bidders-esg_w)
esg_c = df_ESG['deal_identifier'].unique()
esg_c = len(esg_c)


#%%

#################################### 0.3 Return data

#%%
# download a .csv file from CRSP with the WRDS path:  
# Home > Get Data > CRSP > Annual Update > Stock / Security Files > Monthly Stock File

# Get all of the variables holding period stock return (RET), distribution event code (DISTCD), delisting code (DLSTCD), 
# date of delisting payment (DLPDT), amount after delisting (DLAMT), delisting return (DLRET), and CRSP value-weighted index returns (VWRETD)
# Timeframe: Jan 1982 - Dec 2021

#%%
# read return data

df15 = pd.read_csv(file5)


#%% 

# Sort the DataFrame by Bid ID and Date Effective
df14 = df14.sort_values(['deal_identifier', 'Date Effective'])

# Group the DataFrame by deal_identifier and forward fill the "Date Effective" column
df14['Deal Close'] = df14.groupby('deal_identifier')['Date Effective'].fillna(method='ffill')

# Back fill the "Deal Close" column for the losers
df14['Deal Close'] = df14.groupby('deal_identifier')['Deal Close'].fillna(method='bfill')

#%%
from datetime import timedelta
# merge the data to the dataframe with the bids

#%%

# number of years to evaluate post-M&A performance
# for robustness tests vary this number
y = 1

#%%

## Add interval for which the return data is needed to the deal data
df16 = df14[['final_PERMNO', 'Deal Value', 'SDC Deal No', 'Date Announced_x', 'Deal Close','Winner_Dummy','ESG_Dummy', 'ESG Acquiror', 'ESG Target']]

# number of days in three years plus half of a month
interval = 365*y + 15

# create start point of the three year interval
df16['start_date'] = df16['Date Announced_x'] - timedelta(days = interval)

# create end point of the three year interval

def calculate_end_date(row):
    if pd.notnull(row['Deal Close']):
        return row['Deal Close'] + timedelta(days=interval)
    else:
        return row['Date Announced_x'] + timedelta(days=interval)

df16['end_date'] = df16.apply(calculate_end_date, axis=1)

#%%


df15.rename(columns = {'PERMNO':'final_PERMNO'}, inplace = True)

df15['final_PERMNO'] = df15['final_PERMNO'].astype(str)
df16['final_PERMNO'] = df16['final_PERMNO'].astype(str)

df18 = df15.merge(df16, how='outer', on = 'final_PERMNO')

#%%
df18.rename(columns = {'Date Announced_x':'Date_Announced'}, inplace = True)
df18.date = pd.to_datetime(df18.date)

#%%

# filter for data within the specified timeframe
df18 = df18.query('date >= start_date and date <= end_date')

#%%
# join back to original date range table and drop key column
# df19 = df16.merge(df18, on=['start_date','end_date'], how='left').fillna('')
df19 = df16.merge(df18, on=['SDC Deal No'], how='left').fillna('')

# only keep the rows in which the PERMNO codes equal each other

df19 = df19[df19['final_PERMNO_x'] == df19['final_PERMNO_y']]

#%%
# prepare data 

df19 = df19.reset_index(drop=True)

df20 = df19

#%%
df20.rename(columns = {'Date_Announced':'Date Announced'}, inplace = True)
df20.rename(columns = {'start_date_x':'start_date'}, inplace = True)
df20.rename(columns = {'end_date_x':'end_date'}, inplace = True)

# watch out: WRDS adds B/C to the return column, which is why it cannot be directly converted to float
df20['RET'] = df20['RET'].replace('B', '')
df20['RET'] = df20['RET'].replace('C', '')
df20 = df20.replace('', np.nan)

# drop duplicates in delisting returns. Sometimes the same value is repeated in the data
mask = df20['DLRET'].duplicated(keep='first')
df20 = df20[~mask | df20['DLRET'].isnull()]

df20['RET'] = df20['RET'].astype('float')


#%%
# Step 1: Calculate the abnormal returns for each firm 1 year BEFORE and 1 year AFTER the merger contest
## Return - Benchmark 

df20['RET'] = pd.to_numeric(df20['RET'], errors='coerce')
df20['vwretd'] = pd.to_numeric(df20['vwretd'], errors='coerce')
df20['DLRET'] = pd.to_numeric(df20['DLRET'], errors='coerce')
df20['date'] = pd.to_datetime(df20['date'])
df20['Date Announced'] = pd.to_datetime(df20['Date Announced'])
df20['date'] = pd.to_datetime(df20['date'])

#%%

## if delisiting return, take that, otherwise calculate with the normal return

df20['AbnRet'] = np.where(df20['DLRET'].notnull(), df20['DLRET'] - df20['vwretd'], df20['RET'] - df20['vwretd'])

df20.rename(columns = {'final_PERMNO_y':'PERMNO'}, inplace = True)

#%%

df_ret = pd.pivot_table(df20, values=['RET','DLRET'], index='SDC Deal No', aggfunc='count')


#%%
# CALCULATE RETURNS
# Calculate cumulative BHAR

# create Before/After Deal Announcement Dummy 

df20['After_Dummy'] = np.where(df20['date'] >= df20['Date Announced'], 1, 0)

df51 = df20

#%%
# Create ESG Dummy for the Target
df51['ESG_Dummy_t'] = df51['ESG Target_y'].notnull().astype(int)

#%%
#####################

#%%

# BHAR for the whole time
##### Whole analysis is only for after 2002 

df51 = df51[df51['Date Announced_x'].dt.year >= 2002]

# sort the data by 'SDC Deal No' column
df51 = df51.sort_values(['SDC Deal No','date'])

grouped = df51.groupby('SDC Deal No')

# cumulate the simple returns

cumulative_return = df51.groupby('SDC Deal No')['AbnRet'].apply(lambda x: (1 + x).prod() - 1)

# Reset the index of the series and convert it to a DataFrame
df52 = cumulative_return.reset_index()

# Rename the columns of the resulting DataFrame
df52 = df52.rename(columns={'index': 'Index', 'Values': 'Data'})

#%%
# BHAR AFTER the Deal and AFTER 2002

# Create a boolean mask that selects rows where the date is after the announcement date
mask = df51['date'] >= df51['Date Announced']

# Apply the mask to the DataFrame to create the subset
df_after = df51[mask]

#%%

df_ret = pd.pivot_table(df_after, values=['RET','DLRET'], index='SDC Deal No', aggfunc='count')


#%%

# sort the data by 'SDC Deal No' column
df_after = df_after.sort_values(['SDC Deal No','date'])

grouped = df_after.groupby('SDC Deal No')

cumulative_return_after = grouped['AbnRet'].apply(lambda x: (1 + x).prod() - 1)

# Reset the index of the Series and convert it to a DataFrame
df53 = cumulative_return_after.reset_index()

# Rename the columns of the resulting DataFrame
df53 = df53.rename(columns={'index': 'Index', 'Values': 'Data'})

#%%
# Merge cumulative BHAR with the initial dataframe

df30.rename(columns = {'SDC Deal No_x':'SDC Deal No'}, inplace = True)
df52.rename(columns = {'SDC Deal No_x':'SDC Deal No'}, inplace = True)
df53.rename(columns = {'SDC Deal No_x':'SDC Deal No'}, inplace = True)

df30['SDC Deal No'] = df30['SDC Deal No'].astype(str)
df53['SDC Deal No'] = df53['SDC Deal No'].astype(str)

#%%

df54 = pd.merge(df30, df53, on = 'SDC Deal No', how = 'left')

#%%
df54 = pd.merge(df54, df52, on = 'SDC Deal No', how = 'left')

#%%

# Rename the columns
df54 = df54.rename(columns={'AbnRet_x': 'BHAR'})
df54 = df54.rename(columns={'AbnRet_y': 'BHAR_after'})

#%%

df_ret = df54.dropna(subset = ['BHAR_after','Deal Value','OpIncome','Deal Participation','Market Leverage', 'Q','Friendly_Dummy', 'Industry_Dummy'])

#%% 


ret_bids = int(df_ret[['SDC Deal No']].count())
ret_bidders = df_ret['final_PERMNO_x'].unique()
ret_bidders = len(ret_bidders)
ret_w = int(df_ret[['Winner_Dummy']].sum())
ret_l = int(ret_bidders-ret_w)
ret_c = df_ret['deal_identifier'].unique()
ret_c = len(ret_c)

#%%

# Create ESG Dummy for the Acquiror
df54['ESG_Dummy'] = df54['ESG Acquiror'].notnull().astype(int)

# Create ESG Dummy for the Target
df54['ESG_Dummy_t'] = df54['ESG Target'].notnull().astype(int)


df54['Friendly_Dummy'] = np.where(df54['Deal Attitude'] == 'Friendly', 1, 0)

df54['ESG_Score_Winner'] = df54['ESG Acquiror'] * df54['Winner_Dummy']

df54['ESG_Dummy_Winner'] =  df54['ESG_Dummy'] * df54['Winner_Dummy']


#%%

df54['Announcement Year'] = pd.to_numeric(df54['Announcement Year'])


#%%

# Count the number of observations with ESG and Accounting Data and sufficient return data
df59 = df54.dropna(subset=['BHAR_after','ESG_Dummy_t','Deal Value','OpIncome','Deal Participation','Market Leverage','Q','Friendly_Dummy', 'Industry_Dummy'])

fd_bids = int(df59[['SDC Deal No']].count())
fd_bidders = df59['final_PERMNO_x'].unique()
fd_bidders = len(fd_bidders)
fd_w = int(df59[['Winner_Dummy']].sum())
fd_l = int(fd_bidders-fd_w)
fd_c = df59['deal_identifier'].unique()
fd_c = len(fd_c)


#%% 

####################    1 Regressions - Why an ESG score matters

#%%
#%%

####################    1.1 Firms with ESG scores participate in larger transactions and have a larger deal size

#%%

# regress Acquiror ESG Score to Target ESG Score

df60 = df54.dropna(subset=['ESG_Dummy','Deal Value','OpIncome','Deal Participation','Market Leverage', 'Q','Friendly_Dummy', 'Industry_Dummy'])

x = df60[['ESG_Dummy','OpIncome','Deal Participation','Market Leverage', 'Q','Friendly_Dummy', 'Industry_Dummy']]
y = df60['Deal Value']

x = sm.add_constant(x)

model = sm.OLS(y, x)

results_OLS_1_1_a = model.fit()

print(results_OLS_1_1_a.summary())

#%%
# regress Acquiror ESG Score to Target ESG Score

df60 = df54.dropna(subset=['ESG_Dummy_t','Deal Value','OpIncome','Deal Participation','Market Leverage', 'Q','Friendly_Dummy', 'Industry_Dummy'])

x = df60[['ESG_Dummy_t','OpIncome','Deal Participation','Market Leverage', 'Q','Friendly_Dummy', 'Industry_Dummy']]
y = df60['Deal Value']

x = sm.add_constant(x)

model = sm.OLS(y, x)

results_OLS_1_1_t = model.fit()

print(results_OLS_1_1_t.summary())

#%%

# regress Acquiror ESG Score to Target ESG Score

df60 = df54.dropna(subset=['ESG_Dummy','ESG_Dummy_t','Deal Value','OpIncome','Deal Participation','Market Leverage', 'Q','Friendly_Dummy', 'Industry_Dummy'])

x = df60[['ESG_Dummy','ESG_Dummy_t','OpIncome','Deal Participation','Market Leverage', 'Q','Friendly_Dummy', 'Industry_Dummy']]
y = df60['Deal Value']

x = sm.add_constant(x)

model = sm.OLS(y, x)

results_OLS_1_1_c = model.fit()

print(results_OLS_1_1_c.summary())

#%%

## Create Stata output
dfoutput_1_1 = summary_col([results_OLS_1_1_c,results_OLS_1_1_a, results_OLS_1_1_t], stars=True)
print(dfoutput_1_1)


#%%

####################    1.2	Bidders with an ESG Score are more likely to bid for targets with an ESG Score

#%%
#run regression ESG Dummy Target ~ ESG Dummy Acquiror

df60 = df54.dropna(subset=['Deal Value','OpIncome','Deal Participation','Market Leverage', 'Q'])

x_other = df60[['ESG_Dummy','Deal Value','OpIncome','Deal Participation','Market Leverage', 'Q','Friendly_Dummy', 'Industry_Dummy']]
x_year = df60.filter(regex = 'year_')
x = pd.concat([x_other], axis = 1)

y = df60['ESG_Dummy_t']
   
# building the model and fitting the data
ESG_t_a_reg = sm.Logit(y, x).fit()

print(ESG_t_a_reg.summary())

## Create Stata output
dfoutput_1_2 = summary_col([ESG_t_a_reg], stars=True)
print(dfoutput_1_2)


#%% 
####################    1.3	Acquirors with an ESG score are more likely to win a bidding contest

#%%

#run regression Winner_Dummy ~ ESG Dummy Acquiror

df60 = df54.dropna(subset=['ESG_Dummy','Deal Value', 'OpIncome','Deal Participation', 'Market Leverage', 'Q','Friendly_Dummy', 'Industry_Dummy', 'Winner_Dummy'])

x_other = df60[['ESG_Dummy', 'Deal Value', 'OpIncome','Deal Participation', 'Market Leverage', 'Q','Friendly_Dummy', 'Industry_Dummy']]
x_year = df60.filter(regex='year_')
x = pd.concat([x_other], axis=1)

y = df60['Winner_Dummy']

# building the model and fitting the data
ESG_winner_reg = sm.Logit(y, x).fit()

print(ESG_winner_reg.summary())

## Create Stata output
dfoutput_1_3 = summary_col([ESG_winner_reg], stars=True)
print(dfoutput_1_3)


#%%

####################    2 Regressions - Why a high ESG score matters

#%%

####################    2.1 Firms with high ESG scores participate in larger transactions and have larger deal sizes

#%%

# The larger the bid, the higher the ESG Score of the target

df60 = df54.dropna(subset=['ESG Target','Deal Value','OpIncome','Deal Participation','Market Leverage', 'Q','Friendly_Dummy', 'Industry_Dummy'])

x = df60[['ESG Target','OpIncome','Deal Participation','Market Leverage', 'Q','Friendly_Dummy', 'Industry_Dummy']]
y = df60['Deal Value']

# Specify the regression model with contest fixed effects
model_yfe = sm.OLS(y, sm.add_constant(x))

# Fit the regression model
results_OLS_size_t = model_yfe.fit()

print(results_OLS_size_t.summary())

###

#%%

# The larger the bid, the higher the ESG Score of the acquiror

df60 = df54.dropna(subset=['ESG Acquiror','Deal Value','OpIncome','Deal Participation','Market Leverage', 'Q','Friendly_Dummy', 'Industry_Dummy'])

x = df60[['ESG Acquiror','OpIncome','Deal Participation','Market Leverage', 'Q','Friendly_Dummy', 'Industry_Dummy']]
y = df60['Deal Value']

# Specify the regression model with contest fixed effects
model_yfe = sm.OLS(y, sm.add_constant(x))

# Fit the regression model
results_OLS_size_a = model_yfe.fit()

print(results_OLS_size_a.summary())

#%%

# The larger the bid, the higher the ESG Score of the acquiror and the target

df60 = df54.dropna(subset=['ESG Acquiror','ESG Target', 'Deal Value','OpIncome','Deal Participation','Market Leverage', 'Q','Friendly_Dummy', 'Industry_Dummy'])

x = df60[['ESG Acquiror','ESG Target','OpIncome','Deal Participation','Market Leverage', 'Q','Friendly_Dummy', 'Industry_Dummy']]
y = df60['Deal Value']

# Specify the regression model with contest fixed effects
model_yfe = sm.OLS(y, sm.add_constant(x))

# Fit the regression model
results_OLS_size = model_yfe.fit()

print(results_OLS_size.summary())

#%%

# Summary Output

dfoutput = summary_col([results_OLS_size,results_OLS_size_a, results_OLS_size_t], stars=True)
print(dfoutput)


#%%

# The larger the bid, the higher the ESG Score of the acquiror

df60 = df54.dropna(subset=['ESG Target','Deal Value','OpIncome','Deal Participation','Market Leverage', 'Q','Friendly_Dummy', 'Industry_Dummy'])

x = df60[['Deal Value','OpIncome','Deal Participation','Market Leverage', 'Q','Friendly_Dummy', 'Industry_Dummy']]
y = df60['ESG Target']

# Specify the regression model with contest fixed effects
model_yfe = sm.OLS(y, sm.add_constant(x))

# Fit the regression model
results_OLS_size = model_yfe.fit()

print(results_OLS_size.summary())


#%% 

####################    2.2 Bidders with a high ESG score are more likely to bid for targets with a high ESG score

#%% how much of Target ESG Score can be explained by acquiror ESG Score? 

### Full Regression
df_ESG_check = df30.dropna(subset=['ESG Acquiror','ESG Target', 'Deal Value','OpIncome','Deal Participation','Market Leverage', 'Q','Industry_Dummy','Friendly_Dummy'])

x_other = df_ESG_check[['ESG Acquiror', 'Deal Value','OpIncome','Deal Participation','Market Leverage', 'Q','Industry_Dummy','Friendly_Dummy']]
x_year = df_ESG_check.filter(regex='year_')
x = pd.concat([x_other], axis=1)

y = df_ESG_check['ESG Target']

x = sm.add_constant(x)

model_2_2 = sm.OLS(y, x)

results_OLS_2_2 = model_2_2.fit()

print(results_OLS_2_2.summary())


#%%

####################    2.3 Acquirors with a higher ESG score are more likely to win a bidding contest

#%% 

df_ESG = df30.dropna(subset=['ESG Acquiror','Deal Value','OpIncome', 'Deal Participation','Market Leverage', 'Q','Friendly_Dummy','Industry_Dummy'])

#Regressions
# defining the dependent and independent variables

# define variables
x_other_ESG = df_ESG[['ESG Acquiror','Deal Value','OpIncome','Deal Participation','Market Leverage', 'Q','Friendly_Dummy','Industry_Dummy']]
#x_contest = df_ESG.filter(regex='contest_')
x_year = df_ESG.filter(regex='year_')
x_ESG = pd.concat([x_other_ESG], axis=1)
#x_ESG = df_ESG[['ESG Acquiror','MV','OpIncome','Deal Participation','Market Leverage', 'Q','Friendly_Dummy','Industry_Dummy']]
y_ESG = df_ESG['Winner_Dummy']
   
# building the model and fitting the data
log_reg_ESG = sm.Logit(y_ESG, x_ESG).fit()

print(log_reg_ESG.summary())

#%%

################## E Regression

# defining the dependent and independent variables
x_E_other = df_E[['E Score','Deal Value','OpIncome','Deal Participation','Market Leverage', 'Q','Friendly_Dummy','Industry_Dummy']]
x_year_E = df_E.filter(regex='year_')
x_E = pd.concat([x_E_other], axis=1)

y_E = df_E['Winner_Dummy']
   
# building the model and fitting the data
log_reg_E = sm.Logit(y_E, x_E).fit()

print(log_reg_E.summary())

#%%

################## S Regression

# defining the dependent and independent variables
x_S_other = df_S[['S Score','Deal Value','OpIncome','Deal Participation','Market Leverage', 'Q','Friendly_Dummy','Industry_Dummy']]
x_year = df_E.filter(regex='year_')
x_S = pd.concat([x_S_other], axis=1)

y_S = df_S['Winner_Dummy']
   
# building the model and fitting the data
log_reg_S = sm.Logit(y_S, x_S).fit()

print(log_reg_S.summary())


#%%
################## G Regression

# defining the dependent and independent variables
x_G = df_G[['G Score','Deal Value','OpIncome','Deal Participation','Market Leverage', 'Q','Friendly_Dummy','Industry_Dummy']]
y_G = df_G['Winner_Dummy']
   
# building the model and fitting the data
log_reg_G = sm.Logit(y_G, x_G).fit()

print(log_reg_G.summary())

#%% produce output for above regressions

dfoutput = summary_col([log_reg_ESG, log_reg_E, log_reg_S, log_reg_G], stars=True)
print(dfoutput)

#%% look at the descriptive statistics of target's ESG Scores

# Target ESG Score
df30['ESG Target'] = pd.to_numeric(df30['ESG Target'])

t_df = df30.dropna(subset=['ESG Target'])

target_stats = df30['ESG Target'].describe()
target_stats = pd.DataFrame(t_df['ESG Target'].describe())

#%%

### ROBUSTNESS TEST
### Full Regression
df_ESG_check = df30.dropna(subset=['ESG Acquiror','ESG Target', 'MV','OpIncome','Deal Participation','Market Leverage', 'Q','Industry_Dummy','Friendly_Dummy'])

x_check = df_ESG_check[['ESG Acquiror', 'MV','OpIncome','Deal Participation','Market Leverage', 'Q','Industry_Dummy','Friendly_Dummy']]
y_check = df_ESG_check['ESG Target']

x_check = sm.add_constant(x_check)

model = sm.OLS(y_check, x_check)

results = model.fit(cov_type='HC3')

print(results.summary())

#%%

####################    3 Regressions - ESG targets and M&A performance

#%% 

df59 = df54.dropna(subset=['BHAR_after','ESG_Dummy_t','Deal Value','OpIncome','Deal Participation','Market Leverage','Q','Friendly_Dummy', 'Industry_Dummy'])

# only keep winning firms

df59 = df59.loc[df59['Winner_Dummy'] == 1]

# Regression with year fixed effects

q_low_return = df59['BHAR_after'].quantile(0.05)
q_high_return = df59['BHAR_after'].quantile(0.95)

df59 = df59[(df59['BHAR_after']>= q_low_return) & (df59['BHAR_after'] <= q_high_return)]

#define variables
x_other = df59[['ESG_Dummy_t', 'Deal Value','OpIncome', 'Deal Participation','Market Leverage','Q','Friendly_Dummy', 'Industry_Dummy']]

x_year= df59.filter(regex='year_')

x = pd.concat([x_other, x_year], axis=1)
#x = x_other
y = df59['BHAR_after']

# Specify the regression model with contest fixed effects
model_yfe = sm.OLS(y, sm.add_constant(x))

# Fit the regression model
results_OLS_yfe = model_yfe.fit()

print(results_OLS_yfe.summary())

## Create Stata output
dfoutput_3_1 = summary_col([results_OLS_yfe], stars=True)
print(dfoutput_3_1)


#%%
########################### Plots to compare the performance 
print(df51[df51['ESG_Dummy_y'] == 1])

#%%
######################################### create graphs

df59['Trimmed_sample'] = 1

#%%

# merge trimmed sample dummy to the large dataset by SDC Deal No
df51 = df51.merge(df59[['SDC Deal No', 'Trimmed_sample']], on='SDC Deal No', how='left')

# drop all of the rows that are not from the trimmed sample

df51 = df51.dropna(subset=['Trimmed_sample'])


#%%

# Create periods for the monthly return data to be able to group them 
# Deal Identifier

periods = list(range(-13, 13))

# Calculate the number of days to announcement
df51['days_to_announcement'] = (df51['Date Announced'] - df51['date']).dt.days

#df51['days_to_announcement_before'] = (df51['start_date'] - df51['date']).dt.days
#df51['days_to_announcement_after'] = (df51['end_date'] - df51['date']).dt.days


df51['months_to_announcement'] = df51['days_to_announcement']/30 

# Map the days to announcement to the periods of interest
df51['period'] = pd.cut(df51['months_to_announcement'], bins=periods, labels=periods[1:])

print(periods)


#%%
#######################
# Calculate Returns for target-firms with ESG and without ESG

#%%

# Only keep the winners, as we are not interested in the performance of the losers for this analysis
df51 = df51.loc[df51['Winner_Dummy_y'] == 1]

#%%

# calculate return path for firms with and without ESG Score
wESG_df_adj = df51.loc[df51['ESG_Dummy_t'] == 1]
woESG_df_adj = df51.loc[df51['ESG_Dummy_t'] == 0]

# calculate the average monthly return for winners and losers
wESG_average_return_adj = wESG_df_adj.groupby('period')['AbnRet'].mean()
woESG_average_return_adj = woESG_df_adj.groupby('period')['AbnRet'].mean()

#%%
# caculate the cumulative average monthly return for winners and losers from the beginning of the period
wESG_cumulative_return_adj = (1 + wESG_average_return_adj).cumprod() - 1
woESG_cumulative_return_adj = (1 + woESG_average_return_adj).cumprod() - 1

# Set the midpoint of the period
midpoint = len(wESG_average_return_adj)//2 

# Calculate the cumulative returns at the midpoint
cumulative_return_at_midpoint1 = wESG_cumulative_return_adj[midpoint]
cumulative_return_at_midpoint2 = woESG_cumulative_return_adj[midpoint]

# subtract the returns of the midpoint from the cumulative returns
wESG_cumulative_return_adj_mid = wESG_cumulative_return_adj - cumulative_return_at_midpoint1
woESG_cumulative_return_adj_mid = woESG_cumulative_return_adj - cumulative_return_at_midpoint2

# print the average cumulative return path for winners and losers
print('Average cumulative return path for firms with ESG Score:\n', wESG_cumulative_return_adj_mid)
print('Average cumulative return path for firms without ESG Score:\n', woESG_cumulative_return_adj_mid)


#%%
# plot return paths

fig1, ax1 = plt.subplots(figsize=(10,6))

# Plot return paths
wESG_cumulative_return_adj_mid.plot(ax=ax1, label='ESG Target', color = 'black' )
woESG_cumulative_return_adj_mid.plot(ax=ax1, label='non-ESG Target', color = 'grey')

# Set axis labels and title
ax1.set_xlabel('Period')
ax1.set_ylabel('Cumulative Return')
ax1.set_title('Average Cumulative Return Path for Acquirors')

# Add legend
ax1.legend(loc='upper left')

# Add line in the middle
plt.axvline(x = midpoint, color = 'black', label = 'Period 0', linestyle= '--')

# Set axis limits
ax1.set_xlim([-2, 26])
ax1.set_ylim([-0.2, 0.2])

# Set the x-axis tick locator in intervals of 4
ax1.xaxis.set_major_locator(ticker.MultipleLocator(4))

# Set the x-axis tick labels to display the actual numbers
ax1.set_xticklabels([str(i) for i in range(-16, 13, 4)])

#%%

####################### 4 Descriptive Statistics

#%% 

####################### 4.1 ESG statistics

#%%
# ESG 
ESG_stats = df_ESG['ESG Acquiror'].describe()

ESG_df_winners = df_ESG[df_ESG['Winner_Dummy'] == 1]
ESG_df_losers = df_ESG[df_ESG['Winner_Dummy'] == 0]

ESG_stats_winners = pd.DataFrame(ESG_df_winners['ESG Acquiror'].describe())
ESG_stats_winners.rename(columns={'ESG Acquiror': 'ESG Score Winners'}, inplace = True)

ESG_stats_losers = pd.DataFrame(ESG_df_losers['ESG Acquiror'].describe())
ESG_stats_losers.rename(columns={'ESG Acquiror': 'ESG Score Losers'}, inplace = True)

ESG_stats = pd.concat([ESG_stats_winners, ESG_stats_losers], axis = 1)

# Environmental 
E_stats = df_E['S Score'].describe()

E_df_winners = df_E[df_E['Winner_Dummy'] == 1]
E_df_losers = df_E[df_E['Winner_Dummy'] == 0]


E_stats_winners = pd.DataFrame(E_df_winners['E Score'].describe())
E_stats_winners.rename(columns={'E Score': 'E Score Winners'}, inplace = True)

E_stats_losers = pd.DataFrame(E_df_losers['E Score'].describe())
E_stats_losers.rename(columns={'E Score': 'E Score Losers'}, inplace = True)

ESG_stats = pd.concat([ESG_stats, E_stats_winners, E_stats_losers], axis = 1)

# Social
S_stats = df_S['S Score'].describe()

S_df_winners = df_S[df_S['Winner_Dummy'] == 1]
S_df_losers = df_S[df_S['Winner_Dummy'] == 0]


S_stats_winners = pd.DataFrame(S_df_winners['S Score'].describe())
S_stats_winners.rename(columns={'S Score': 'S Score Winners'}, inplace = True)

S_stats_losers = pd.DataFrame(S_df_losers['S Score'].describe())
S_stats_losers.rename(columns={'S Score': 'S Score Losers'}, inplace = True)

ESG_stats = pd.concat([ESG_stats, S_stats_winners, S_stats_losers], axis = 1)


# Governance
G_stats = df_G['S Score'].describe()

G_df_winners = df_G[df_G['Winner_Dummy'] == 1]
G_df_losers = df_G[df_G['Winner_Dummy'] == 0]

G_stats_winners = pd.DataFrame(G_df_winners['G Score'].describe())
G_stats_winners.rename(columns={'G Score': 'G Score Winners'}, inplace = True)

G_stats_losers = pd.DataFrame(G_df_losers['G Score'].describe())
G_stats_losers.rename(columns={'G Score': 'G Score Losers'}, inplace = True)

ESG_stats = pd.concat([ESG_stats, G_stats_winners, G_stats_losers], axis = 1)


#%% 
# correlation matrix 
df_ESG_corr = df30[['ESG Acquiror', 'E Score', 'S Score', 'G Score']]
corr_matrix_ESG = df_ESG_corr.corr()

#%% 

# perform mean difference test for ESG

# perform t-test for ESG
t_stat_ESG, p_value_ESG = stats.ttest_ind(ESG_df_winners['ESG Acquiror'], ESG_df_losers['ESG Acquiror'])
print("t-statistic ESG:", t_stat_ESG)
print("p-value ESG:", p_value_ESG)

# perform t-test for E
t_stat_E, p_value_E = stats.ttest_ind(E_df_winners['E Score'], E_df_losers['E Score'])
print("t-statistic E:", t_stat_E)
print("p-value E:", p_value_E)

# perform t-test for S
t_stat_S, p_value_S = stats.ttest_ind(S_df_winners['S Score'], S_df_losers['S Score'])
print("t-statistic S:", t_stat_S)
print("p-value S:", p_value_S)

# perform t-test for G
t_stat_G, p_value_G = stats.ttest_ind(G_df_winners['G Score'], G_df_losers['G Score'])
print("t-statistic G:", t_stat_G)
print("p-value G:", p_value_G)


#%% 

# correlation matrix for controls

df_controls_corr = df30[['Deal Value','OpIncome','Deal Participation','Market Leverage', 'Q','Friendly_Dummy','Industry_Dummy']]
corr_matrix_controls = df_controls_corr.corr()
    
    
#%% write summary statistics in excel 

# import xlsxwriter module
ESG_stats.to_excel('ESG_statistics.xlsx', index=True)
corr_matrix_ESG.to_excel('ESG correlation matrix.xlsx', index=True)
corr_matrix_controls.to_excel('Controls correlation matrix.xlsx', index=True)

#%% 

####################### 4.2 Winner and Loser statistics

#%%
# Firm characteristics: Variables such as firm size, age, and financial performance can be important controls in bidding contests as they may influence a firm's ability to submit competitive bids.
# 
# Industry factors: Variables such as market concentration, competition, and industry growth rates may impact bidding outcomes and can be important control variables in regression analysis.
# 
# Bid characteristics: Variables such as bid amount, number of bids, and timing of the bid submission may affect bidding outcomes and can be included as controls in the regression analysis.
# 
# ESG-specific variables: Other ESG-related variables such as environmental impact, social responsibility, and governance practices can also be included as control variables in the regression analysis.

# 
# - Tobin’s Q is the ratio of Market value to book value of assets.
# - PP&E is book value of property, plant and equipment divided by total assets. 
# - Profitability is operating income before depreciation divided by total assets.

# In[ ]:
#%%
df60 = df54.dropna(subset=['Deal Value','OpIncome','Deal Participation','Market Leverage', 'Q'])

#%%
## Winners AT

df61 = df60
df61 = df61.reset_index()  

df61['Winner_AT'] = np.where(df61['Winner_Dummy'] == 1, df61['AT'], float('NaN'))
df61.dropna(subset = ['Winner_AT'], inplace = True)

#mean
Winner_AT_mean = round(df61['Winner_AT'].mean(),2)

#median 
Winner_AT_med = round(df61['Winner_AT'].median(),2)

#std
Winner_AT_std = round(df61['Winner_AT'].std(),2)

#N
Winner_AT_N = len(df61['Winner_AT'])


#%%
## Losers AT

df61 = df60
df61 = df61.reset_index()  

df61['Loser_AT'] = np.where(df61['Winner_Dummy'] == 0, df61['AT'], float('NaN'))
df61.dropna(subset = ['Loser_AT'], inplace = True)
#mean
Loser_AT_mean = round(df61['Loser_AT'].mean(),2)

#median 
Loser_AT_med = round(df61['Loser_AT'].median(),2)

#std
Loser_AT_std = round(df61['Loser_AT'].std(),2)

#N
Loser_AT_N = len(df61['Loser_AT'])


#%%

#SALEQ

## Winners SALE
df61 = df60
df61 = df61.reset_index()   

df61['Winner_SALE'] = np.where(df61['Winner_Dummy'] == 1, df61['SALE'], float('NaN'))
df61.dropna(subset = ['Winner_SALE'], inplace = True)

#mean
Winner_SALE_mean = round(df61['Winner_SALE'].mean(),2)

#median 
Winner_SALE_med = round(df61['Winner_SALE'].median(),2)

#std
Winner_SALE_std = round(df61['Winner_SALE'].std(),2)

#N
Winner_SALE_N = len(df61['Winner_SALE'])

## Losers SALE

df62 = df60
df62 = df62.reset_index()  

df62['Loser_SALE'] = np.where(df62['Winner_Dummy'] == 0, df62['SALE'], float('NaN'))
df62.dropna(subset = ['Loser_SALE'], inplace = True)

Loser_SALE_mean = round(df62['Loser_SALE'].mean(),2)

#median 
Loser_SALE_med = round(df62['Loser_SALE'].median(),2)

#std
Loser_SALE_std = round(df62['Loser_SALE'].std(),2)

#N
Loser_SALE_N = len(df62['Loser_SALE'])

#%%
##########################################
# Winner Market Leverage

winner_market_leverage = df60[['Market Leverage','Winner_Dummy']]
winner_market_leverage['Winner_ML'] = np.where(winner_market_leverage['Winner_Dummy'] == 1, winner_market_leverage['Market Leverage'], float('NaN'))
winner_market_leverage.dropna(subset = ['Winner_ML'], inplace = True)

# mean
winner_market_leverage_mean = round(winner_market_leverage['Market Leverage'].mean(),2)

#median 
winner_market_leverage_med = round(winner_market_leverage['Market Leverage'].median(),2)

#std
winner_market_leverage_std = round(winner_market_leverage['Market Leverage'].std(),2)

#N
winner_market_leverage_N = len(winner_market_leverage['Market Leverage'])

#%%

# Loser Market Leverage
loser_market_leverage = df60[['Market Leverage','Winner_Dummy']]
loser_market_leverage['Loser_ML'] = np.where(loser_market_leverage['Winner_Dummy'] == 0, loser_market_leverage['Market Leverage'], float('NaN'))
loser_market_leverage.dropna(subset = ['Loser_ML'], inplace = True)

# mean
loser_market_leverage_mean = round(loser_market_leverage['Market Leverage'].mean(),2)

#median 
loser_market_leverage_med = round(loser_market_leverage['Market Leverage'].median(),2)

#std
loser_market_leverage_std = round(loser_market_leverage['Market Leverage'].std(),2)

#N
loser_market_leverage_N = len(loser_market_leverage['Market Leverage'])



#%%
##########################################
# Book Leverage

df60['Book Leverage'] = df60['DT'] / df60['AT']

## Winner Book Leverage

winner_book_leverage = df60[['Book Leverage','Winner_Dummy']]

winner_book_leverage['Winner_BL'] = np.where(winner_book_leverage['Winner_Dummy'] == 1, winner_book_leverage['Book Leverage'], float('NaN'))
winner_book_leverage.dropna(subset = ['Winner_BL'], inplace = True)

# mean
winner_book_leverage_mean = round(winner_book_leverage['Book Leverage'].mean(),2)

#median 
winner_book_leverage_med = round(winner_book_leverage['Book Leverage'].median(),2)

#std
winner_book_leverage_std = round(winner_book_leverage['Book Leverage'].std(),2)

#N
winner_book_leverage_N = len(winner_book_leverage['Book Leverage'])


#%%

## Loser Book Leverage

loser_book_leverage = df60[['Book Leverage','Winner_Dummy']]

loser_book_leverage['Loser_BL'] = np.where(loser_book_leverage['Winner_Dummy'] == 0, loser_book_leverage['Book Leverage'], float('NaN'))
loser_book_leverage.dropna(subset = ['Loser_BL'], inplace = True)

# mean
loser_book_leverage_mean = round(loser_book_leverage['Book Leverage'].mean(),2)

#median 
loser_book_leverage_med = round(loser_book_leverage['Book Leverage'].median(),2)

#std
loser_book_leverage_std = round(loser_book_leverage['Book Leverage'].std(),2)

#N
loser_book_leverage_N = len(loser_book_leverage['Book Leverage'])


#%%
##########################################
# Operating Income

## Winner OpIncome as % of Total Assets

winner_opincome = pd.pivot_table(df60, values='OpIncome', index=['SDC Deal No','Winner_Dummy'], aggfunc='mean')
winner_opincome = winner_opincome.reset_index()  

winner_opincome['Winner_OI'] = np.where(winner_opincome['Winner_Dummy'] == 1, winner_opincome['OpIncome'], float('NaN'))
winner_opincome.dropna(subset = ['Winner_OI'], inplace = True)

# mean
winner_opincome_mean = round(winner_opincome['OpIncome'].mean(),2)

#median 
winner_opincome_med = round(winner_opincome['OpIncome'].median(),2)

#std
winner_opincome_std = round(winner_opincome['OpIncome'].std(),2)

#N
winner_opincome_N = len(winner_opincome['OpIncome'])

#%%

loser_opincome = pd.pivot_table(df60, values='OpIncome', index=['SDC Deal No','Winner_Dummy'], aggfunc='mean')
loser_opincome = loser_opincome.reset_index()  

loser_opincome['Loser_OI'] = np.where(loser_opincome['Winner_Dummy'] == 0, loser_opincome['OpIncome'], float('NaN'))
loser_opincome.dropna(subset = ['Loser_OI'], inplace = True)

# mean
loser_opincome_mean = round(loser_opincome['OpIncome'].mean(),2)

#median 
loser_opincome_med = round(loser_opincome['OpIncome'].median(),2)

#std
loser_opincome_std = round(loser_opincome['OpIncome'].std(),2)

#N
loser_opincome_N = len(loser_opincome['OpIncome'])


#%% 
##########################################
# Q

# Winner Q

winner_Q = pd.pivot_table(df60, values='Q', index=['SDC Deal No','Winner_Dummy'], aggfunc='mean')
winner_Q = winner_Q.reset_index()  

winner_Q['winner_Q'] = np.where(winner_Q['Winner_Dummy'] == 1, winner_Q['Q'], float('NaN'))
winner_Q.dropna(subset = ['winner_Q'], inplace = True)

# mean
winner_Q_mean = round(winner_Q['Q'].mean(),2)

#median 
winner_Q_med = round(winner_Q['Q'].median(),2)

#std
winner_Q_std = round(winner_Q['Q'].std(),2)

#N
winner_Q_N = len(winner_Q['Q'])

#%%

# Loser Q

loser_Q = pd.pivot_table(df60, values='Q', index=['SDC Deal No','Winner_Dummy'], aggfunc='mean')
loser_Q = loser_Q.reset_index()  

loser_Q['loser_Q'] = np.where(loser_Q['Winner_Dummy'] == 0, loser_Q['Q'], float('NaN'))
loser_Q.dropna(subset = ['loser_Q'], inplace = True)

# mean
loser_Q_mean = round(loser_Q['Q'].mean(),2)

#median 
loser_Q_med = round(loser_Q['Q'].median(),2)

#std
loser_Q_std = round(loser_Q['Q'].std(),2)

#N
loser_Q_N = len(loser_Q['Q'])

#%%
##########################################
# Winner MV

winner_MV = pd.pivot_table(df60, values='MV', index=['SDC Deal No','Winner_Dummy'], aggfunc='mean')
winner_MV = winner_MV.reset_index()  

winner_MV['Winner_MV'] = np.where(winner_MV['Winner_Dummy'] == 1, winner_MV['MV'], float('NaN'))
winner_MV.dropna(subset = ['Winner_MV'], inplace = True)

# mean
winner_MV_mean = round(winner_MV['MV'].mean(),2)

#median 
winner_MV_med = round(winner_MV['MV'].median(),2)

#std
winner_MV_std = round(winner_MV['MV'].std(),2)

#N
winner_MV_N = len(winner_MV['MV'])

#%%

# Loser MV

loser_MV = pd.pivot_table(df60, values='MV', index=['SDC Deal No','Winner_Dummy'], aggfunc='mean')
loser_MV = loser_MV.reset_index()  

loser_MV['Loser_MV'] = np.where(loser_MV['Winner_Dummy'] == 0, loser_MV['MV'], float('NaN'))
loser_MV.dropna(subset = ['Loser_MV'], inplace = True)

# mean
loser_MV_mean = round(loser_MV['MV'].mean(),2)

#median 
loser_MV_med = round(loser_MV['MV'].median(),2)

#std
loser_MV_std = round(loser_MV['MV'].std(),2)

#N
loser_MV_N = len(loser_MV['MV'])

#%%
##########################################
# Winner Table

winner_values = {'Winners':['Total Assets [$m]', 'Market Value [$m]', 'Net Revenue [$m]', 'Tobins Q', 'Book Leverage', 'Market Leverage','Profitability'], 'Mean':[Winner_AT_mean, winner_MV_mean, Winner_SALE_mean, winner_Q_mean, winner_book_leverage_mean,winner_market_leverage_mean, winner_opincome_mean], 'Median': [Winner_AT_med, winner_MV_med, Winner_SALE_med,winner_Q_med,winner_book_leverage_med, winner_market_leverage_med,winner_opincome_med], 'Std':[Winner_AT_std, winner_MV_std, Winner_SALE_std,winner_Q_std, winner_book_leverage_std, winner_market_leverage_std, winner_opincome_std], 'N':[Winner_AT_N, winner_MV_N, Winner_SALE_N, winner_Q_N, winner_book_leverage_N, winner_market_leverage_N, winner_opincome_N]}

winner_table = pd.DataFrame(winner_values).set_index('Winners')

winner_table


#%%

# Loser Table

loser_values = {'Losers':['Total Assets [$m]', 'Market Value [$m]', 'Net Revenue [$m]', 'Tobins Q', 'Book Leverage', 'Market Leverage','Profitability'], 'Mean':[Loser_AT_mean, loser_MV_mean, Loser_SALE_mean, loser_Q_mean, loser_book_leverage_mean,loser_market_leverage_mean, loser_opincome_mean], 'Median': [Loser_AT_med, loser_MV_med, Loser_SALE_med,loser_Q_med,loser_book_leverage_med, loser_market_leverage_med,loser_opincome_med], 'Std':[Loser_AT_std, loser_MV_std, Loser_SALE_std,loser_Q_std, loser_book_leverage_std, loser_market_leverage_std, loser_opincome_std], 'N':[Loser_AT_N, loser_MV_N, Loser_SALE_N, loser_Q_N, loser_book_leverage_N, loser_market_leverage_N, loser_opincome_N]}

loser_table = pd.DataFrame(loser_values).set_index('Losers')

loser_table

#%%

#get descriptive statistics for winners and losers BHAR

#q_low_return = df60['BHAR_after'].quantile(0.05)
#q_high_return = df60['BHAR_after'].quantile(0.95)

#df60 = df60[(df60['BHAR_after']>= q_low_return) & (df60['BHAR_after'] <= q_high_return)]


BHAR_after_stats = df60['BHAR_after'].describe()

BHAR_after_winners = df60[df60['Winner_Dummy'] == 1]
BHAR_after_losers = df60[df60['Winner_Dummy'] == 0]

BHAR_after_winners = pd.DataFrame(BHAR_after_winners['BHAR_after'].describe())
BHAR_after_winners.rename(columns={'BHAR_after': 'BHAR_after_winners'}, inplace = True)

BHAR_after_losers = pd.DataFrame(BHAR_after_losers['BHAR_after'].describe())
BHAR_after_losers.rename(columns={'BHAR_after': 'BHAR_after_losers'}, inplace = True)

BHAR_stats = pd.concat([BHAR_after_winners, BHAR_after_losers], axis = 1)

#%%
########################## 4.3 Industry statistics

#%%

# Creating the pivot tables for firm counts
pivot_table_industry_acquiror = pd.pivot_table(df60, index='Acquiror Macro Industry', aggfunc='size')
pivot_table_industry_acquiror.name = 'Acquiror Firms'  # Assigning a name to the Series

pivot_table_industry_target = pd.pivot_table(df60, index='Target Macro Industry', aggfunc='size')
pivot_table_industry_target.name = 'Target Firms'  # Assigning a name to the Series

# Merging the pivot tables by industry
pivot_table_industry = pd.merge(pivot_table_industry_acquiror, pivot_table_industry_target, left_index=True, right_index=True, how='outer')

############ ESG Dummy

# Summing up the ESG_Dummy variable per industry
esg_dummy_sum_per_industry_acquiror = df60.groupby('Acquiror Macro Industry')['ESG_Dummy'].sum()

# Adding the sum of ESG_Dummy to the merged pivot table
pivot_table_industry['ESG_Dummy Acquiror'] = esg_dummy_sum_per_industry_acquiror

# Summing up the ESG_Dummy variable per industry
esg_dummy_sum_per_industry_target = df60.groupby('Acquiror Macro Industry')['ESG_Dummy_t'].sum()

# Adding the sum of ESG_Dummy to the merged pivot table
pivot_table_industry['ESG_Dummy Target'] = esg_dummy_sum_per_industry_target


############ ESG Scores
# Calculating the mean Acquiror ESG score per industry
mean_esg_score_per_industry_acquiror = df60.groupby('Acquiror Macro Industry')['ESG Acquiror'].mean()

# Calculating the mean Target ESG score per industry
mean_esg_score_per_industry_target = df60.groupby('Target Macro Industry')['ESG Target'].mean()

# Adding the mean ESG score to the merged pivot table
pivot_table_industry['Mean ESG Score Acquiror'] = mean_esg_score_per_industry_acquiror

# Adding the mean ESG score to the merged pivot table
pivot_table_industry['Mean ESG Score Target'] = mean_esg_score_per_industry_target

# Saving the merged table to Excel
pivot_table_industry.to_excel('industry_descriptives.xlsx', index=True)

#%%

mean_ESG_score_acquiror = df60['ESG Acquiror'].mean()
mean_ESG_score_target = df60['ESG Target'].mean()


#%%

########################## 4.4 Deal statistics

#%%

# Deal Characteristics

# Deal Characteristics
# Tender Offer, hostile, percentage in stock, cash, number of bidders, contest duration


#%%

# Deal Value 

deal_value = pd.pivot_table(df60, values='Deal Value (USD, millions)', index=['SDC Deal No','Winner_Dummy'], aggfunc='mean')

deal_value = deal_value.reset_index()  

deal_value['Deal Value (USD, millions)'] = np.where(deal_value['Winner_Dummy'] == 1, deal_value['Deal Value (USD, millions)'], float('NaN'))
deal_value.dropna(subset = ['Deal Value (USD, millions)'], inplace = True)

# p25
deal_value_25 = round(np.percentile(deal_value['Deal Value (USD, millions)'], 25),2)

# median
deal_value_med = round(np.median(deal_value['Deal Value (USD, millions)']),2)

# p75
deal_value_75 = round(np.percentile(deal_value['Deal Value (USD, millions)'], 75),2)

# mean
deal_value_mean = round(deal_value['Deal Value (USD, millions)'].mean(),2)

#std
deal_value_std = round(deal_value['Deal Value (USD, millions)'].std(),2)

#N
deal_value_N = len(deal_value['Deal Value (USD, millions)'])


#%%

# Percentage paid in Stock
df60.rename(columns={'Percentage of Stock': 'Stock_pct'}, inplace=True)

stock_pct = pd.pivot_table(df60, values='Stock_pct', index=['SDC Deal No','Winner_Dummy'], aggfunc='mean')
stock_pct = stock_pct.reset_index()  

stock_pct['Stock_pct'] = np.where(stock_pct['Winner_Dummy'] == 1, stock_pct['Stock_pct'], float('NaN'))
stock_pct.dropna(subset = ['Stock_pct'], inplace = True)

# p25
stock_pct_25 = round(np.percentile(stock_pct['Stock_pct'], 25),2)

# median
stock_pct_med = round(np.median(stock_pct['Stock_pct']),2)

# p75
stock_pct_75 = round(np.percentile(stock_pct['Stock_pct'], 75),2)

# mean
stock_pct_mean = round(stock_pct['Stock_pct'].mean(),2)

#std
stock_pct_std = round(stock_pct['Stock_pct'].std(),2)

#N
stock_pct_N = len(stock_pct['Stock_pct'])


#%%
    
#Percentage of Cash

df60.rename(columns={'Percentage of Cash': 'Cash_pct'}, inplace=True)

# Percentage paid in Cash

df60.rename(columns={'Percentage of Cash': 'Cash_pct'}, inplace=True)

cash_pct = pd.pivot_table(df60, values='Cash_pct', index=['SDC Deal No','Winner_Dummy'], aggfunc='mean')
cash_pct = cash_pct.reset_index()  

cash_pct['Cash_pct'] = np.where(cash_pct['Winner_Dummy'] == 1, cash_pct['Cash_pct'], float('NaN'))
cash_pct.dropna(subset = ['Cash_pct'], inplace = True)

# p25
cash_pct_25 = round(np.percentile(cash_pct['Cash_pct'], 25),2)

# median
cash_pct_med = round(np.median(cash_pct['Cash_pct']),2)

# p75
cash_pct_75 = round(np.percentile(cash_pct['Cash_pct'], 75),2)

# mean
cash_pct_mean = round(cash_pct['Cash_pct'].mean(),2)

#std
cash_pct_std = round(cash_pct['Cash_pct'].std(),2)

#N
cash_pct_N = len(cash_pct['Cash_pct'])


# In[ ]:

# Number of Bidders

bidder_no = pd.pivot_table(df60, values='Number of Bidders', index=['SDC Deal No','Winner_Dummy'], aggfunc='mean')
bidder_no = bidder_no.reset_index()  

bidder_no['bidder_no'] = np.where(bidder_no['Winner_Dummy'] == 1, bidder_no['Number of Bidders'], float('NaN'))
bidder_no.dropna(subset = ['bidder_no'], inplace = True)

# p25
bidder_no_25 = round(np.percentile(bidder_no['bidder_no'], 25),2)

# median
bidder_no_med = round(np.median(bidder_no['bidder_no']),2)

# p75
bidder_no_75 = round(np.percentile(bidder_no['bidder_no'], 75),2)

# mean
bidder_no_mean = round(bidder_no['bidder_no'].mean(),2)

#std
bidder_no_std = round(bidder_no['bidder_no'].std(),2)

#N
bidder_no_N = len(bidder_no['bidder_no'])


# In[ ]:

# Contest Duration
# create a column with the difference between date announced and date effective in days

N = 30
df60['timedelta'] = df60['Date Effective'] - df60['Date Announced_y']

df60['days'] = df60['timedelta'].dt.days

df60['Contest_Duration'] = np.where(df60['Winner_Dummy'] == 1, df60['days'],float('NaN')) 

contest_dur = df60.dropna(subset = ['Contest_Duration'])

# p25
contest_dur_25 = round(np.percentile(contest_dur['Contest_Duration']/N, 25),2)

# median
contest_dur_med = round(np.median(contest_dur['Contest_Duration'])/N,2)

# p75
contest_dur_75 = round(np.percentile(contest_dur['Contest_Duration']/N, 75),2)

# mean
contest_dur_mean = round(contest_dur['Contest_Duration'].mean()/N,2)

#std
contest_dur_std = round((contest_dur['Contest_Duration']/N).std(),2)

#N
contest_dur_N = len(contest_dur['Contest_Duration'])


#%%

# Deal Characteristics Table
deal_values = {'Deal':['Deal Value','Percentage paid in stock','Percentage paid in cash','Number of Bidders','Contest Duration [months]'], 'P25':[deal_value_25,stock_pct_25,cash_pct_25,bidder_no_25,contest_dur_25], 'Median': [deal_value_med, stock_pct_med,cash_pct_med,bidder_no_med,contest_dur_med], 'P75':[deal_value_75, stock_pct_75, cash_pct_75,bidder_no_75,contest_dur_75],'Mean':[deal_value_mean, stock_pct_mean,cash_pct_mean,bidder_no_mean,contest_dur_mean], 'Std':[deal_value_std, stock_pct_std, cash_pct_std,bidder_no_std,contest_dur_std], 'N':[deal_value_N, stock_pct_N, cash_pct_N,bidder_no_N,contest_dur_N]}

deal_table = pd.DataFrame(deal_values).set_index('Deal')


#%%

# Graph for Contests over time

df60['Year'] = np.where(df60['Winner_Dummy'] == 1, df60['Date Effective'].dt.year,float('NaN')) 
df60['Year'] = df60['Year'].astype('float')

year_contests = df60.dropna(subset = ['Year'])


year_contests_pivot = pd.pivot_table(year_contests, values='SDC Deal No', index='Year', aggfunc='count')
year_contests_pivot.reset_index(drop=False, inplace = True)
year_contests_pivot.rename(columns = {'SDC Deal No':'Number of contests'}, inplace = True)

year_contests_pivot['Year'] = year_contests_pivot['Year'].astype(int)

# Create a new figure and axes for the second graph
fig2, ax2 = plt.subplots(figsize=(10, 6))

bar_plot = sns.barplot(x = 'Year',
            y = 'Number of contests',
            data = year_contests_pivot,
           color = 'black', ax=ax2)

for index, label in enumerate(bar_plot.get_xticklabels()):
   if index % 5 == 0:
      label.set_visible(True)
   else:
      label.set_visible(False)